/*
 * data aquisition example. simulated interrupts.
 */
#ifndef daq_h
#define daq_h

#define DAQ_IR_FREQ_DEFAULT	0
#define DAQ_IR_MAX_PROXY	5
#define DAQ_IR_MAX_READ_Q	20
#define DAQ_IR_READ_BUF_SIZE	512
#define DAQ_IR_CUR_CHAR		0x20
#define DAQ_IR_LOW_FILTER	0x20
#define DAQ_IR_HIGH_FILTER	0x7e

typedef struct dqDev dqDev;
typedef struct dqQ dqQ;
typedef struct dqQEntry dqQEntry;
typedef struct dqReadBuf dqReadBuf;

struct dqReadBuf
{
	char buf[DAQ_IR_READ_BUF_SIZE];
	int head;
	int tail;
	int len;
	int overFlow;
};

struct dqQEntry
{
	void	  *data;
	dqQEntry *next;
	dqQEntry *prev;
};

struct dqQ 
{
	dqQEntry *first;
	dqQEntry *last;
	int	  count;
};

struct dqProcWait 
{
	pid_t	pid;
	long	nbytes;
	long	writeOffset;
};


struct dqDev
{
	int irFreq;
	int irFreqInterimCnt;
	unsigned char curChar;
	char lowFilter;
	char highFilter;
	int overFlow;
	dqReadBuf buf[2];
	int curIrReadBuf;
	int oldIrReadBuf;
	dqQ inputQ;
	dqQ procWaitQ;
};

struct dqIrProxy
{
	pid_t 	 proxy;
	dqDev   *dev;
	int	hintid;
	
};


pid_t far DQHint( void );
int DQHintProxy(pid_t p);

int DQAddQ(struct dqQ *q, void *data);
void *DQRemQ(struct dqQ *q);
void *DQPeekQ(struct dqQ *q);
int DQCopyFromQToReply(pid_t p, long nbytes, long *offset, dqReadBuf *buf);
void *DQDetachQ(struct daqQ *q, void *fn(void *, void *), void *key);
void *DQCmpFn(void *data, void *key);

int DQRead(pid_t p, file_t *f, long nbytes);
int DQClose(pid_t p, file_t *f);
int DQQioctl(pid_t p, file_t *f, int request, long lenOfData,
		void *data, long maxBytesRead);
int DQCreateDev(inode_t **ip);


char *DQTimeBaseSync();
int DQStartTime();
int DQStopTime(double *secDiff);

#endif
