
/*
 *  hint.c
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <libc.h>
#include <sys/io_msg.h>
#include <sys/inline.h>
#include <sys/irqinfo.h>
#include <sys/kernel.h>
#include <sys/proxy.h>
#include <sys/trace.h>
#include <sys/tracecod.h>
#include "iomgr.h"
#include "daq.h"

extern struct dqIrProxy DQIrProxyList[];
extern int DQIrProxyCount;
extern char *DQProgname;

static int dqLastHint=0;

static double dqDiffTime;	/* Used only in a verbose mode */

/* 
 * For QNX to operate at its best, this interrupt handler should 
 * be as short as possible. 
 * This Interrupt handler is shared between multiple daq devices.
 * Find the current device.
 * 
 */
pid_t far DQHint( void )
{
	dqDev *dev;
	dqReadBuf *buf;
	pid_t retProxy=0;


	/* Alternate between devices if more than 1. */
	dqLastHint = dqLastHint < DQIrProxyCount-1 ? dqLastHint+1 : 0;

	dev = DQIrProxyList[dqLastHint].dev;

	dev->irFreqInterimCnt++;

	if (dev->irFreq <= dev->irFreqInterimCnt)
	{
		dev->irFreqInterimCnt = 0;
		dev->curChar++;
		dev->curChar = dev->curChar > DAQ_IR_HIGH_FILTER ?
			DAQ_IR_LOW_FILTER : dev->curChar ;

		if (dev->curChar >= dev->lowFilter &&
			dev->curChar <= dev->highFilter)
		{
			buf = &dev->buf[dev->curIrReadBuf];
			if (buf->head >= buf->len-1)
			{
				/* Swing to other buffer */
				if (dev->oldIrReadBuf != 2)
				{
					/* Old buf not clear. */
					buf->overFlow = 1;
					buf->head = buf->len-1;
					
				}
				else
				{
					/* Switch buffers */
					dev->oldIrReadBuf = dev->curIrReadBuf;
					dev->curIrReadBuf = !dev->curIrReadBuf;
					buf = &dev->buf[dev->curIrReadBuf];
				}
/*
				Trace2b( _TRACE_TEMPORARY|10, _TRACE_TRANSIENT,
					2, 1, strlen(progname)+1,
					progname);
*/
				/* Trigger the proxy */
				retProxy = DQIrProxyList[dqLastHint].proxy;
			}
			buf->buf[buf->head++] = dev->curChar;
		}
	}


	return retProxy ;
}
