
/* 
 * This proxy program will support the proxy generated by the 
 * daq interrupt handler.
 * 
 */

#include <libc.h>
#include "iomgr.h"
#include "daq.h"
#include <i86.h>

/* Some global variables to share with the interrupt handler */
extern struct dqIrProxy DQIrProxyList[];
extern int DQIrProxyCount;
extern int DQVerbose;

int DQHintProxy(pid_t p)
{
	int     i;
	dqDev *dev;
	dqReadBuf *buf=0;
	dqReadBuf *oldBuf;
	struct dqProcWait *waitQ;

	for (i=0; i<DQIrProxyCount && DQIrProxyList[i].proxy != p; i++)
		;
	if (DQIrProxyList[i].proxy != p)
		return 0;

	dev = DQIrProxyList[i].dev;

	/* Get the data */
        if (dev->oldIrReadBuf < 2)
	{
		/* Setup a pointer to the data */
		oldBuf = &dev->buf[dev->oldIrReadBuf];

		/* A Process is currently waiting for data */
		if ((waitQ = DQPeekQ(&dev->procWaitQ)) != 0)
		{
			if (DQCopyFromQToReply(waitQ->pid, waitQ->nbytes, 
			    &waitQ->writeOffset, oldBuf) == 0)
			{
				/* Process has been replied to */
				waitQ = DQRemQ(&dev->procWaitQ);
			}
		}

		/* 
		 * Check if there is still some data left in the
		 * buffer. If so then copy it to the inputQ.
		 */
		if (oldBuf->tail <= oldBuf->head)
		{
			if ((buf = malloc(sizeof(dqReadBuf))) == 0) 
			{
				/* No memory */
				if (DQVerbose) printf("Failed allocating buf in proxy.c\n");
				return 0;
			}
			else
			{

				/* Q the data */
				if (dev->inputQ.count < 20)
				{
					memcpy(buf, oldBuf,sizeof(dqReadBuf)); 
					DQAddQ(&dev->inputQ, buf);
				}
				else
				{
					free(buf);
					dev->overFlow++;
				}
			}
		}
		/* 
		* Mark the oldIrReadBuf to be free for the 
		* interrupt handler to reuse.
		*/
		oldBuf->head = oldBuf->tail = oldBuf->overFlow = 0;
		dev->oldIrReadBuf = 2;
	}
	return 1;
}


