
/* 
 * 
 */

#include <libc.h>
#include "iomgr.h"
#include "daq.h"



int DQAddQ(struct dqQ *q, void *data)
{
	dqQEntry *entry;
	dqQEntry *oldEntry;

	if ((entry = calloc(sizeof(dqQEntry),1)) == 0)
	{
		/* Malloc failed */
		return ENOMEM; 
	}
	entry->data = data;
	if (q->first == 0)
	{
		q->first = entry;
		q->last = entry;
	}
	else
	{
		oldEntry = q->last;
		oldEntry->next = entry;
		entry->prev = oldEntry;
		entry->next = 0;
		q->last = entry;
	}
	q->count++;
	return EOK;
}

void *DQRemQ(struct dqQ *q)
{
	dqQEntry *entry;
	void *data;


	if (q->first)
	{
		/* D Q the entry */
		entry = q->first;
		if (entry->next)
		{
			entry->next->prev = 0;
			q->first = entry->next;
		}
		else
		{
			q->first = 0;
			q->last = 0;
		}
		q->count--;
		data = entry->data;
		free(entry);
		return (data);
	}
	else
	{
		return 0;
	}
}

void *DQPeekQ(struct dqQ *q)
{

	if (q->first)
	{
		return q->first->data;
	}
	else
	{
		return NULL;
	}
}

void *DQDetachQ(struct dqQ *q, void *fn(void *, void *), void *key)
{
	dqQEntry *entry;
	void *retData=NULL;

	entry =  q->first;

	while (entry  && retData == NULL)
	{
		if (entry->data)
		{
			if ((retData = fn(entry->data, key)) != NULL)
			{
				q->count--;
				if (q->first == entry)
				{
					q->first = entry->next;
				}
				if (q->last == entry)
				{
					q->last = entry->prev;
				}
				if (entry->next)
				{
					entry->next->prev = entry->prev;
				}
				if (entry->prev)
				{
					entry->prev->next = entry->next;
				}
				free(entry);
			}
			else
			{
				entry = entry->next;
			}
			
		}
		else
		{
			entry = entry->next;
		}
	}

	return retData;
}
