
/* 
 * This proxy program will support the read function for the daq
 * I/O Manager.
 * 
 */

#include <libc.h>
#include <sys/kernel.h>
#include <sys/io_msg.h>
#include "iomgr.h"
#include "daq.h"

extern int DQVerbose;


int DQRead(pid_t p, file_t *f, long nbytes)
{
	dqDev 	*dev = f->inode->ino_data;
	struct dqProcWait *proc;
	struct dqReadBuf *buf;
	long offset=0;
	int qProc = 1;
	

	if (DQVerbose) printf("Process Id: %d is requesting a read of %d bytes\n",
		p, nbytes);
	

	buf = DQPeekQ(&dev->inputQ);

	while (buf && qProc)
	{
		if (DQCopyFromQToReply(p, nbytes, &offset, buf) == 0)
		{
			qProc = 0;
		}

		if (buf->tail >= buf->head)
		{
			/* Remove used buf */
			buf = DQRemQ(&dev->inputQ);
			free(buf);
			/* look at next buf */
			buf = DQPeekQ(&dev->inputQ);
		}
	}


	if (qProc)
	{
		if (nonblock(f)) 
		{
			if (DQVerbose) printf("Driver would block on read\n");
			reply_errno(p, EWOULDBLOCK);
			return;
		}
		if ((proc=malloc(sizeof(struct dqProcWait))) == 0) 
		{
			reply_errno(p, ENOMEM);
			return;
		}

		/* Add the process id to the proc wait for input q. */
		proc->pid = p;
		proc->nbytes = nbytes;
		proc->writeOffset = offset;
		if (DQAddQ(&dev->procWaitQ, proc) != EOK)
		{
			reply_errno(p, ENOMEM);
			return;
		}
	} 
	return;
}

int DQCopyFromQToReply(pid_t p, long nbytes, long *offset, dqReadBuf *buf)
{
	long toWrite = nbytes - *offset;
	struct _io_read_reply reply;

	/* 
	 * The head is an index to the last char. tail is the
	 * index to the first char.
	 */
	toWrite = toWrite > (buf->head+1) - buf->tail ? 
		(buf->head+1) - buf->tail : toWrite;

	/* 
	 * Add the contents from buf->buf at tail to the waiting 
	 * processes (blocked) receive message buffer.
	 */
	Writemsg(p, (*offset+(sizeof(reply)-sizeof(reply.data))), 
			&buf->buf[buf->tail], toWrite);

	*offset += toWrite;

	/* Adjust tail pointer of the read buffer */
	buf->tail += toWrite;

	if (*offset >= nbytes)
	{
		
		reply.status = EOK;
		reply.zero = 0;
		reply.nbytes = nbytes;
		/* 
		 * This buf will contain the first part of the message 
		 * Reply to the blocked process.
		 */
		Reply(p, &reply, sizeof(reply)-sizeof(reply.data));
		return 0;
	}
	else
	{
		return 1;
	}
}
