
/*
 *  iorecv.c
 *
 *  This module is skeleton code for simple I/O manager which
 *  receives message containing text and prints out what it gets.
 *  The writer is in writer.c.
 *
 *  Run it as: iorecv -pdirectory_for_device device_name
 *  Example: iorecv -p/mydevices devicea
 *
 *  1997 03 03  S. Dufresne     created
*/

#include <libc.h>
#include "iomgr.h"

#include <sys/io_msg.h>
#include <sys/kernel.h>
#include <sys/uio.h>

/* 
 * pid_t is the sending process.
 * file_t is a pointer to the file_t struct.
 * n is the number of bytes written
 * buf is a pointer to the receive buffer.
 * len is the number of bytes written to the receive buffer.
 */
static
iorecv_write(pid_t p, file_t *f, unsigned n, void *buf, unsigned len)
{
	struct _io_write_reply w;
	char *bufPtr=0;
	char *outputPtr;
    /*
     * fill this in with the code for reading what was received and
     * displaying it to the screen
     */
	if (n > len)
	{
		len = n;
		/* allocate a buffer large enough to hold the message */
		if ((bufPtr = malloc(len)) == 0)
		{
			printf("--- Malloc failed\n");
			w.status = ENOMEM;
			Reply(p, &w.status, sizeof(w.status));
			return;
		}
		/* Get the entire message */
		if (wcopyin(p, 0, bufPtr, len) != len)
		{
			printf("--- wcopyin failed\n");
			w.status = errno;
			Reply(p, &w.status, sizeof(w.status));
			free(bufPtr);
			return;
		}
		outputPtr = bufPtr;
	}
	else
	{
		/* Setup a pointer to print out the message */
		outputPtr = buf;
	}

	/* Print the received data to stdout */
	outputPtr[n] = 0;
	printf("%s\n",outputPtr);
	if (bufPtr != 0)
	{
		free(bufPtr);
	}
	w.status = EOK;
	w.nbytes = n;
	w.zero = 0;
	Reply(p, &w, sizeof(w));
}

create_dev(inode_t **ip)
{
	inode_t         *io;
	static iofuns   iorecvclass;

	if (!iorecvclass.write) {
        iorecvclass.write = iorecv_write;
	}

	if ((io=inode_alloc(&iorecvclass)) == 0) {
		return ENOMEM;
	}
	iorecvclass.minor++;
	io->ino_ftype |= S_IFCHR;
	*ip = io;
	return EOK;
}
