
/*
 *  reader.c
 *
 *  This will try to read from the given io manager.  It is set up to
 *  handle SIGUSR1 and so can be used to test the io manager proc_killed()
 *  function.
 *
 *  Run it as: reader directory_for_device/device_name
 *  Example: reader /mydevices/devicea
 *
 *  1997 05 06  S. Dufresne     created
*/

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

/*
 *  our global variables.
*/

char    *progname = "reader";

void handler( int signo );

main( int argc, char *argv[] )
{
    char    buf[20];
    int     fd;
    
    if ( argc != 2 ) {
        printf( "use: reader directory_for_device/device_name\n" );
        printf( "Example: reader /mydevices/devicea\n" );
        exit( EXIT_FAILURE );
    }

    signal( SIGUSR1, handler );
    
    if ( (fd = open( argv[1], O_RDONLY )) == -1 ) {
        printf( "%s: could not open %s: %s\n",
                progname, argv[1], strerror(errno) );
        exit( EXIT_FAILURE );
    }
    
    if ( read( fd, buf, sizeof(buf) ) == -1 ) {
        printf( "%s: could not read: %s\n",
                progname, strerror(errno) );
        exit( EXIT_FAILURE );
    }

    close( fd );
}

void handler( int signo )
{
    signo = signo;
}
