/*-
 * An example of an io manager which puts just a prefix in the
 * prefix tree and then allows reads and writes from it.
 *
 */


#include <libc.h>
#include <sys/kernel.h>
#include <sys/stat.h>
#include "iointernal.h"
#include "iomessage.h"

int verbose;
char    *progname = "null";
static int ndevs = 8;
char      *dpath = "/testing";

char msg[256];

fatalerr(const char *fmt, ...)
{
	va_list	v;
	va_start(v, fmt);
	write(STDERR_FILENO, progname, strlen(progname));
	write(STDERR_FILENO, ": ", 2);
	vsprintf(msg, fmt, v);
	write(STDERR_FILENO, msg, strlen(msg));
	va_end(v);
	exit(EXIT_FAILURE);
}

void
panic(const char *msg)
{
	sprintf(msg,"%s: panic '%s'\n", progname, msg);
	write(STDERR_FILENO, msg, strlen(msg));
	exit(EXIT_FAILURE);
}

static
sa_write(pid_t p, file_t *f, unsigned n, void *buf, unsigned len)
{
	struct _io_write_reply	w;
	char					*b;

	if ((b=malloc(n+1))==0) {
		reply_errno(p, ENOMEM);
		return;
	}
	if (wcopyin(p, 0, b, n) != n) {
		reply_errno(p, errno);
		free(b);
		return;
	}
	b[n] = NULL;
	printf("%s\n", b);
	free(b);
	w.status = EOK;
	w.nbytes = n;
	w.zero  = 0;
	Reply(p, &w, sizeof w);
}

static
sa_read(pid_t p, file_t *f, long nbytes)
{
	char	*b;
	
	if ((b=malloc(nbytes))==0) {
		reply_errno(p, ENOMEM);
		return;
	}
	if (nbytes > 1)
		memset(b, 'X', nbytes-1);
	b[nbytes-1] = NULL;
	reply_read(p, b, nbytes);
	return;
}

static
sa_lookup(inode_t *dir, char *name, inode_t **result)
{
	*result = dir;
	return EOK;
}

setup()
{
	int       err;
	inode_t   *root;
	int        create_dev(inode_t **ip);
	dev_t      rm;
	static iofuns devfuns;

	if (server_init() == -1) {
		fatalerr("Unable to become server '%s'\n", strerror(errno));
	}
	if ((err=inode_init(ndevs)) != EOK) {
		fatalerr("unable to init inode table (%s)\n", strerror(err));
	}

	if (!devfuns.write) {
		devfuns.write = sa_write;
		devfuns.read = sa_read;
		devfuns.lookup = sa_lookup;
	}
	root = inode_alloc(&devfuns);
	root->ino_ftype = S_IFNAM | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP |
					  S_IROTH | S_IWOTH;
	if ((errno=root_mount(dpath, root)) != EOK) {
		fatalerr("cannot create root dir '%s'\n", strerror(errno));
	}

	if ((rm=getunit(0)) == -1) {
		fatalerr("cannot get root entry\n");
	}
}

main(int argc, char **argv)
{
	int       c;

	progname = argv[0];
	while ((c=getopt(argc, argv, "vn:p:")) != -1) {
		switch (c) {
		case 'v': verbose = 1; break;
		case 'n': ndevs = strtol(optarg,0,0); break;
		case 'p': dpath = optarg; break;
		}
	}

	signal(SIGHUP, SIG_IGN);
	signal(SIGINT, SIG_IGN);
	setup();
	server(); /* does not return */
}
