
/*
 *  writeread.c
 *
 *  This will write the given text out to the given io manager and then
 *  try to read a result back.
 *
 *  Run it as: writeread directory_for_device/device_name text_to_write
 *  Example: writeread /mydevices/devicea "GET VOLTAGE"
 *
 *  1997 05 06  S. Dufresne     created
*/

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

/*
 *  our global variables.
*/

char    *progname = "writeread";

main( int argc, char *argv[] )
{
    char    buf[20];
    int     fd;
    
    if ( argc != 3 ) {
        printf( "use: writeread directory_for_device/device_name text_to_write\n" );
        printf( "Example: writeread /mydevices/devicea \"GET VOLTAGE\"\n" );
        exit( EXIT_FAILURE );
    }

    if ( (fd = open( argv[1], O_RDWR )) == -1 ) {
        printf( "%s: could not open %s: %s\n",
                progname, argv[1], strerror(errno) );
        exit( EXIT_FAILURE );
    }
    
    if ( write( fd, argv[2], strlen(argv[2]) + 1 ) == -1 ) {
        printf( "%s: could not write '%s': %s\n",
                progname, argv[2], strerror(errno) );
        exit( EXIT_FAILURE );
    }

    if ( read( fd, buf, sizeof(buf) ) == -1 ) {
        printf( "%s: could not read: %s\n",
                progname, strerror(errno) );
        exit( EXIT_FAILURE );
    }

    printf( "%s: Wrote \"%s\", Read \"%s\"\n", progname, argv[2], buf );
    
    close( fd );
}
