/*-
  inode_open(): assemble appropriate data structures and invoke the
  class-specific open.
 */

#include <string.h>
#include <sys/fd.h>
#include "iointernal.h"

_open(pid_t pid, mode_t mode, inode_t *p, int fd)
{
	file_t           *f;
	int               n;

	if ((f=falloc()) == 0) {
		reply_errno(pid, ENOMEM);
		return -1;
	}
	memset(f, 0, sizeof *f);
	f->mode = mode;
	f->count++;
	f->inode = p;
	if (FD_SET(pid, fd, f2index(f)) == -1) {
		reply_errno(pid, errno);
		ffree(f);
		return -1;
	}
/*- notify the resource of opens */
	if ((n=f_open(pid,f,mode)) != EOK)  {
		FD_CLEAR(pid, fd);
		ffree(f);
		reply_errno(pid, n);
		return -1;
	}
	reply_errno(pid, EOK);
	return 0;
}
