/*-
 * fd: grungy filedescriptor/qnx system interface.
 *
 *  The mysterious __init_fd(), __get_fd() and __get_pid_info()
 * are covered here and associated with the internal file table
 * structures.
 *  The file structures are hard-coded as limits; it would be
 * nicer to have runtimes.
 */

#include <sys/fd.h>
#include <sys/psinfo.h>
#include <sys/stat.h>
#include <string.h>
#include "iointernal.h"


#ifdef IN_PROC
file_t *
lookup_fd(pid_t pid, int fd, int errok)
{
	struct _fd_entry __far *ftp;
	if ((ftp=get_ftp(pid, fd, 0)) && ftp->handle) {
		return __index2f(ftp->handle);
	} else if (!errok) {
		reply_errno(pid, EBADF);
	}
	return 0;
}

int
FD_SET(pid_t pid, int fd, int handle)
{
	struct _fd_entry __far *ftp;
	if ((ftp=get_ftp(pid, fd, 1)) == 0) {
		return EBADF;
	}
	ftp->handle = handle;
	if (ftp->nid == 0)
		ftp->nid = osinfo.nodename;
	if (ftp->pid == 0)
		ftp->pid = ftp->vid = PROC_PID;
	ftp->flags = ftp->zero1 = ftp->zero2 = 0;
	return EOK;
}



ids(pid_t pid, uid_t *u, gid_t *g)
{
	Proc_t *p;
	if ((p=chk_pid(pid)) == 0)
		return -1;
	*u= p->proc_euid;
	*g= p->proc_egid;
	return 0;
}

isroot(pid_t pid)
{
	Proc_t *p;
	return ((p=chk_pid(pid)) && p->proc_euid==0;
}

#else


file_t *
lookup_fd(pid_t pid, int fd, int errok)
{
	int        indx;
	if ((indx=__get_fd(pid, fd, fd_ctrl)) == -1 || indx == 0) {
		if (!errok)
			reply_errno(pid, EBADF);
		return 0;
	}
	return __index2f(indx);
}

ids(pid_t pid, uid_t *u, gid_t *g)
{
	struct _psinfo3 psdata3;
	__get_pid_info(pid, &psdata3, fd_ctrl);
	*u = psdata3.euid;
	*g = psdata3.egid;
}

isroot(pid_t pid)
{
	uid_t uid; gid_t gid;
	ids(pid, &uid, &gid);
	return uid == 0;
}

#undef FD_SET
int
FD_SET(pid_t pid, int fd, int handle)
{
	qnx_fd_attach(pid, fd, 0,0,0,0, handle);
	return EOK;
}

#endif
