/*-
 * inode allocator.
 * The class defaults are set for class members which are 0.
 * This module has to be kept up to date with the iofuns structure.
 */
#include <stdlib.h>
#include "iointernal.h"


/*-
 * alloc & free inodes
 */
static inode_t    *devlist;
static int        devs = 0;
static int        ldev=0;

inode_t *
inode_alloc(iofuns *class)
{
	inode_t    *p;
	if (ldev >= devs) {
		return 0;
	}
	p = devlist + ldev;
	ldev++;
	/*-
	 * fill in defaults for the class
	 * must be kept in sync with class definition.
	 */
	if (class->major == 0) {
		class->major = getmajor();
	}
	if (!class->open)	class->open  = iodef_open;
	if (!class->close)	class->close = iodef_close;
	if (!class->read)	class->read = iodef_read;
	if (!class->write)	class->write = iodef_write;
	if (!class->creat)	class->creat = iodef_creat;
	if (!class->link)	class->link = iodef_link;
	if (!class->unlink)	class->unlink = iodef_unlink;
	if (!class->lookup)	class->lookup = iodef_lookup;
	if (!class->setsize)	class->setsize = iodef_setsize;
	if (!class->ioctl)	class->ioctl = iodef_ioctl;
	if (!class->qioctl)	class->qioctl = iodef_qioctl;
	p->ino_ino = (long) p;     /* just a number */
	p->ino_size = 0;
	p->ino_dev  = makedev(getnid(), class->major, class->minor);
	p->ino_rdev = makedev(getnid(), class->major, class->minor); 
	p->ino_uid  = 0;
	p->ino_gid  = 0;
	p->ino_atime =
	     p->ino_ctime =
	     p->ino_mtime =
	   /*m->ino_ftime =*/ gettime();
	p->ino_ftype = 0;
	p->ino_nlink = 1;


	p->ino_driver = class;  /* init all to point to class. */
	p->ino_data   = 0;
	return p;
}

int
inode_init(int ndevs)
{
	if ((devlist=calloc(sizeof *devlist, ndevs+1)) == 0)
		return ENOMEM;
	devs = ndevs+1;
	return EOK;
}
