/*-
 * io_chown() - change the owner of a file; two mechanisms are
 * supported based on the value of chown_restricted.
 */

#include "iointernal.h"

extern int chown_restricted;

io_chown(pid_t pid, file_t *f, uid_t uid, gid_t gid)
{
	inode_t    *st = f->inode;
	if (isowner(f) == 0) {
		return reply_errno(pid, EBADF);
	}
	if (chown_restricted) {
		uid_t uuid;
		gid_t ugid;
		ids(pid, &uuid, &ugid);
		if (st->ino_uid != uid && uuid != 0) {
			return reply_errno(pid, EPERM);
		}
		if (st->ino_gid != gid && gid != ugid && uuid!=0) {
			return reply_errno(pid, EPERM);
		}
	}
	st->ino_uid = uid;
	st->ino_gid = gid;
	return reply_errno(pid, EOK);
}
