/*-
 * io_handle() is a weird one; it exists because sometimes programs
 * need to access files which they don't have access to.  This
 * should have been integrated into OPEN.
 */

#include <sys/io_msg.h>
#include <sys/kernel.h>
#include "iointernal.h"

/*-
 * should check extended flags: _IO_EFLAG_DIR - referenced a directory
 *                              _IO_EFLAG_DOT - must be a dir.
 */

io_handle(pid_t pid, mode_t mode, char *path, int fd, inode_t *root, int eflag)
{
	int             e;
	inode_t        *file, *dir;
	char            comp[NAME_MAX+1];
	uid_t          u;
	gid_t          g;

	if ((e=nameip(pid, path, FEXEC, root, &dir, comp)) != EOK ||
	    (e=d_lookup(dir, comp, &file)) != EOK) {
		return reply_errno(pid, e);
	}
	if ((eflag&(_IO_EFLAG_DOT|_IO_EFLAG_DIR)) && 
	     !S_ISDIR(file->ino_ftype)) {
		return reply_errno(pid, ENOTDIR);
	}
	ids(pid, &u, &g);

	switch (mode) {
	case _IO_HNDL_INFO:
		return _open(pid, FNONE, file, fd);

	case _IO_HNDL_RDDIR:
		if ((e=chk_perm(u, g, FREAD, file)) == EOK) {
			if (!S_ISDIR(file->ino_ftype)) {
				return reply_errno(pid, ENOTDIR);
			}
			return _open(pid, FREAD, file, fd);
		} else {
			return reply_errno(pid, e);
		}
		break;
	case _IO_HNDL_CHANGE:
	case _IO_HNDL_UTIME:
		if ((e=chk_perm(u, g, FOWNER, file))==EOK) {
			return _open(pid, FOWNER, file, fd);
		} else {
			return reply_errno(pid, e);
		}
	case _IO_HNDL_LOAD:
	case _IO_HNDL_CLOAD:
		if ((e=chk_perm(u, g, FEXEC, file)) == EOK) {
			return _open(pid, FREAD, file, fd);
		} else {
			return reply_errno(pid, e);
		}
	default:
		return reply_errno(pid, ENOSYS);
	}
	/*NOTREACHED*/
}
