/*-
 * first level of open() - find the file and call _open with the
 * corresponding inode.
 */

#include "iointernal.h"


io_open(pid_t pid, int oflag, char *path, int fd, inode_t *root, mode_t perm)
{
	int            e;
	inode_t       *dir, *n;
	char           comp[NAME_MAX+1];
	mode_t         fmode;
	uid_t          u;
	gid_t          g;

	ids(pid, &u, &g);
	fmode = base_modes(oflag);
	if ((e=nameip(pid, path, FEXEC, root, &dir, comp)) != EOK) {
		return reply_errno(pid, e);
	}
	/*-
	 * Some of the rules here were derived by experimentation;
	 * O_TRUNC is only applied if the open mode is write.
	 */
	if ((e=d_lookup(dir, comp, &n)) == EOK) {
		if ((oflag&(O_CREAT|O_EXCL)) == (O_CREAT|O_EXCL)) {
			return reply_errno(pid, EEXIST);
		}
		if ((e=chk_perm(u, g, fmode, n)) != EOK) {
			return reply_errno(pid, e);
		}
		if (fmode&FWRITE) {
			if (oflag&O_TRUNC) {
				if ((e=d_setsize(n, 0)) != EOK) {
					return reply_errno(pid, e);
				}
			}
		}
		return _open(pid, fmode, n, fd);
	} else if ((oflag&O_CREAT) != O_CREAT) {
		return reply_errno(pid, ENOENT);
	} else {
		if ((e=chk_perm(u, g, FWRITE, dir)) != EOK) {
			return reply_errno(pid, e);
		}
		if ((e=d_creat(dir, &n)) != EOK) {
			return reply_errno(pid, e);
		}
		if ((e=d_link(dir, comp, n)) != EOK) {
			return reply_errno(pid, e);
		}
		n->ino_uid   = u;
		n->ino_gid   = g;
		n->ino_ftype &= ~0777;
		n->ino_ftype |= perm&0777;
		if (_open(pid, fmode, n, fd) == -1) {
			d_unlink(dir, comp);
			return -1;
		}
		return 0;
	}
}
