/*- internal convenience for library */

#ifndef iointernal_h
#define iointernal_h

#include <stddef.h>
#include <stdarg.h>

#include "iomgr.h"

/*-
 * default io functions
 */
extern iodef_init(inode_t *, unsigned majno);
extern iodef_close(file_t *f);
extern iodef_open(proc_t p, file_t *, mode_t m);
extern iodef_read(proc_t pid, file_t *m, void *p, ulong_t nbytes);
extern iodef_write(proc_t pid, file_t *m, unsigned n, void *p, unsigned len);
/*- directory services */
extern iodef_lookup(inode_t *dir, char *name, inode_t **res);
extern iodef_link(inode_t *dir, char *name, inode_t *file);
extern iodef_unlink(inode_t *dir, char *name);
extern iodef_creat(inode_t *dir, inode_t **ip);
extern iodef_setsize(inode_t *dir, ulong_t len);
/*- device services */
extern iodef_ioctl();

extern iodef_qioctl(proc_t pid, file_t *m, unsigned request, unsigned n, 
		void *p, unsigned len);


#define ino_open(i)   ((i)->ino_driver->open)
#define ino_close(i)  ((i)->ino_driver->close)
#define ino_read(i)   ((i)->ino_driver->read)
#define ino_write(i)  ((i)->ino_driver->write)
#define ino_ioctl(i)  ((i)->ino_driver->ioctl)
#define ino_qioctl(i)  ((i)->ino_driver->qioctl)
#define ino_link(i)   ((i)->ino_driver->link)
#define ino_unlink(i)   ((i)->ino_driver->unlink)
#define ino_lookup(i) ((i)->ino_driver->lookup)
#define ino_creat(i)  ((i)->ino_driver->creat)
#define ino_setsize(i)  ((i)->ino_driver->setsize)


/*-
 *   less cumbersome macroes:
 */
extern err_t d_lookup(inode_t *dir, char *name, inode_t **res);
extern err_t d_link(inode_t *dir, char *name, inode_t *file);
extern err_t d_unlink(inode_t *dir, char *name);
extern err_t d_creat(inode_t *, inode_t **ip);
extern err_t d_setsize(inode_t *, ulong_t len);

extern err_t f_open(proc_t p, file_t *f, unsigned mode);
extern err_t f_close(proc_t p, file_t *f);
extern int   f_read(proc_t p, file_t *f, unsigned n);
extern int   f_write(proc_t p, file_t *f, unsigned n, void *buf, unsigned len);
extern int   f_qioctl(proc_t p, file_t *f, unsigned request, unsigned n, void *buf, unsigned len);

#define d_lookup(d,n,r) (*ino_lookup(d))((d),(n),(r))
#define d_link(d,n,f)     (*ino_link(d))((d),(n),(f))
#define d_unlink(d,n)     (*ino_unlink(d))((d),(n))
#define d_creat(d,ip) (*ino_creat(d))((ip))
#define d_setsize(d,len) (*ino_setsize(d))((d),(len))

#define f_open(p,f,m)     (*ino_open(f->inode))((p),(f),(m))
#define f_read(p, f, n)   (*ino_read(f->inode))((p),(f),(n))
#define f_write(p, f, n,b,l)  (*ino_write(f->inode))((p),(f),(n),(b),(l))
#define f_close(p,f)      (*ino_close(f->inode))((p),(f))
#define f_qioctl(p, f, r, n,b,l)  (*ino_qioctl(f->inode))((p),(f),(r), (n),(b),(l))


/*-
 * io-class messages.
 */
extern int io_open(proc_t pid, int oflag, char *path, int fd, inode_t *root, mode_t perm);
extern int io_handle(proc_t pid, int oflag, char *path, int fd, inode_t *root, int eflag);
extern int io_chdir(proc_t pid, char *path, inode_t *root);
extern int io_stat(proc_t pid, char *path, inode_t *root);
extern int io_fstat(proc_t pid, file_t *f);
extern int io_chmod(proc_t pid, file_t *f, mode_t mode);
extern int io_chown(proc_t pid, file_t *f, uid_t uid, gid_t gid);
extern int io_close(proc_t pid, file_t *f);
extern int io_read(proc_t pid, file_t *f, int nbytes);
extern int io_write(proc_t pid, file_t *f, unsigned n, void *b, unsigned len);
extern int io_dup(proc_t pid, file_t *f, proc_t dstpid, int dstfd);
extern int io_utime(proc_t pid, file_t *f, time_t mtime, time_t atime);
extern int io_lseek(proc_t pid, file_t *f, int whence, off_t loc);
extern int io_rewinddir(proc_t pid, file_t *f);
extern int io_readdir(proc_t pid, file_t *f, int ndirs);
extern int io_flags(proc_t pid, char *path, inode_t *root);
/*-
 * fs-class messages
 */
extern int fs_mkdir(pid_t pid, char *path, inode_t *root, mode_t modes);
extern int fs_unlink(pid_t pid, char *path, inode_t *root);


extern int namei(proc_t pid, char *path, mode_t mode, inode_t *root, inode_t **res);
extern int nameip(proc_t pid, char *path, mode_t mode, inode_t *root,
                 inode_t **pptr, char *component);
extern int _open(pid_t pid, mode_t mode, inode_t *p, int fd);
extern mode_t base_modes(mode_t t);

#ifndef IN_PROC
extern struct _fd_ctrl   *fd_ctrl;
#define FD_SET(p,f,x)  qnx_fd_attach((p), (f), 0, 0, 0, 0, (x))
#endif

#define FD_CLEAR(p,f)  FD_SET((p), (f), 0)

extern file_t file_table[];
#define __index2f(n)  (file_table+(n)-1)


#endif
