
#ifndef iomsg_h
#define iomsg_h

#include <sys/types.h>
#include <sys/sys_msg.h>
#include <sys/io_msg.h>
#include <sys/fsys_msg.h>
#include <sys/dev_msg.h>
#include "iointernal.h"

typedef struct {
	struct _sysmsg_hdr                  hdr;
	union {
		struct _sysmsg_death        death;
		struct _sysmsg_signal       sig;
		struct _sysmsg_trace        trace;
		struct _sysmsg_version      version;
	} body;
} sys_msg;

typedef struct {
	struct _sysmsg_hdr_reply 	hdr;
	union {
		/* no death or signal reply */
		struct _sysmsg_trace_reply   trace;
		struct _sysmsg_version_reply version;
		struct _sysmsg_slib_reply    slib;
	} body;
} sys_reply;


typedef union {
	msg_t                               type;
	struct _io_open                     open;
	struct _io_close                    close;
	struct _io_read                     read;
	struct _io_write                    write;
	struct _io_lseek                    lseek;
	struct _io_config                   config;
	struct _io_dup                      dup;
	struct _io_fstat                    fstat;
	struct _io_chmod                    chmod;
	struct _io_chown                    chown;
	struct _io_utime                    utime;
	struct _io_flags                    flags;
	struct _io_lock                     lock;
	struct _io_readdir                  readdir;
	struct _io_rewinddir                rewinddir;
	struct _io_ioctl                    ioctl;
	struct _io_qioctl                   qioctl;
	struct _io_select                   select;
} io_msg;

typedef union {
	msg_t                               status;
	struct _io_open_reply               open;
	struct _io_close_reply              close;
	struct _io_read_reply               read;
	struct _io_write_reply              write;
	struct _io_lseek_reply              lseek;
	struct _io_config_reply             config;
	struct _io_dup_reply                dup;
	struct _io_fstat_reply              fstat;
	struct _io_chmod_reply              chmod;
	struct _io_chown_reply              chown;
	struct _io_utime_reply              utime;
	struct _io_flags_reply              flags;
	struct _io_readdir_reply            readdir;
	struct _io_rewinddir_reply          rewinddir;
	struct _io_ioctl_reply              ioctl;
	struct _io_qioctl_reply             qioctl;
	struct _io_select_reply             select;
} io_reply;

typedef union {
	msg_t                               type;
	struct _fsys_mkspecial              mkspec;
	struct _fsys_remove                 unlink;
	struct _fsys_link                   link;
	struct _fsys_ramdisk                ramdisk;
	struct _fsys_block_read             block_read;
	struct _fsys_block_write            block_write;
	struct _fsys_get_entry              get_entry;
	struct _fsys_sync                   sync;
	struct _fsys_mount_part             mount_part;
	struct _fsys_mount                  mount;
	struct _fsys_get_mount              get_mount;
	struct _fsys_disk_space             disk_space;
	struct _fsys_pipe                   pipe;
	struct _fsys_trunc                  trunc;
	struct _fsys_xstat                  xstat;
	struct _fsys_mount_ext_part         mount_ext_part;
	struct _fsys_umount                 umount;
	struct _fsys_readlink               readlink;
	struct _fsys_driver                 driver;
	struct _fsys_fsync                  fsync;
	struct _fsys_info                   info;
	struct _fsys_fdinfo                 fdinfo;
	struct _fsys_driver32               driver32;
} fs_msg;
typedef union {
	struct _fsys_link_reply             link;
	struct _fsys_ramdisk_reply          ramdisk;
	struct _fsys_block_read_reply       block_read;
	struct _fsys_block_write_reply      block_write;
	struct _fsys_get_entry_reply        get_entry;
	struct _fsys_sync_reply             sync;
	struct _fsys_mount_part_reply       mount_part;
	struct _fsys_mount_reply            mount;
	struct _fsys_get_mount_reply        get_mount;
	struct _fsys_disk_space_reply       disk_space;
	struct _fsys_pipe_reply             pipe;
	struct _fsys_trunc_reply            trunc;
	struct _fsys_xstat_reply            xstat;
	struct _fsys_mount_ext_part_reply   mount_ext_part;
	struct _fsys_umount_reply           umount;
	struct _fsys_readlink_reply         readlink;
	struct _fsys_driver_reply           driver;
	struct _fsys_fsync_reply            fsync;
	struct _fsys_info_reply             info;
	struct _fsys_fdinfo_reply           fdinfo;
	struct _fsys_driver32_reply         driver32;
} fs_reply;



typedef union {
	msg_t                               type;
	struct  _dev_tcgetattr              tcgetattr;
	struct  _dev_tcsetattr              tcsetattr;
	struct  _dev_mode                   mode;
	struct  _dev_tcsendbreak            tcsendbreak;
	struct  _dev_tcdropline             tcdropline;
	struct  _dev_tcdrain                tcdrain;
	struct  _dev_tcflush                tcflush;
	struct  _dev_tcflow                 tcflow;
	struct  _dev_tcgetpgrp              tcgetpgrp;
	struct  _dev_tcsetpgrp              tcsetpgrp;
	struct  _dev_tcsetct                tcsetct;
	struct  _dev_insert_chars           insert_chars;
	struct  _dev_waiting                waiting;
	struct  _dev_info                   info;
	struct  _dev_fdinfo                 fdinfo;
	struct  _dev_arm                    arm;
	struct  _dev_state                  state;
	struct  _dev_read                   read;
	struct  _dev_write                  write;
	struct  _dev_size                   size;
	struct  _dev_readex                 readex;
	struct  _dev_osize                  osize;
	struct  _dev_reset                  reset;
} dev_msg;

typedef union {
	struct  _dev_tcgetattr_reply        tcgetattr;
	struct  _dev_tcsetattr_reply        tcsetattr;
	struct  _dev_mode_reply             mode;
	struct  _dev_tcsendbreak_reply      tcsendbreak;
	struct  _dev_tcdropline_reply       tcdropline;
	struct  _dev_tcdrain_reply          tcdrain;
	struct  _dev_tcflush_reply          tcflush;
	struct  _dev_tcflow_reply           tcflow;
	struct  _dev_tcgetpgrp_reply        tcgetpgrp;
	struct  _dev_tcsetpgrp_reply        tcsetpgrp;
	struct  _dev_tcsetct_reply          tcsetct;
	struct  _dev_insert_chars_reply     insert_chars;
	struct  _dev_waiting_reply          waiting;
	struct  _dev_info_reply             info;
	struct  _dev_fdinfo_reply           fdinfo;
	struct  _dev_arm_reply              arm;
	struct  _dev_state_reply            state;
	struct  _dev_read_reply             read;
	struct  _dev_write_reply            write;
	struct  _dev_size_reply             size;
	struct  _dev_readex_reply           readex;
	struct  _dev_osize_reply            osize;
	struct  _dev_reset_reply            reset;
} dev_reply;

typedef union {
	msg_t           type;
	sys_msg         sys;
	io_msg          io;
	fs_msg          fs;
	dev_msg         dev;
	char            nbuf[PATH_MAX*2+sizeof(struct _io_open)];
} message;

typedef union {
	msg_t           status;
	sys_reply       sys;
	io_reply        io;
	fs_reply        fs;
	dev_reply       dev;
} reply;


extern int sys_message(pid_t pid, sys_msg *sys, unsigned len);
extern int io_message(pid_t pid, io_msg *m, unsigned len);
extern int fs_message(pid_t pid, fs_msg *m, unsigned len);
extern int dev_message(pid_t pid, dev_msg *m, unsigned len);

enum {
	_SYS_CLASS    = 0,
	_IO_CLASS     = 1,
	_FS_CLASS     = 2,
	_DEV_CLASS    = 3
};
#define MSG_CLASS(m) ((m)>>8)





#endif
