
/*- server: main loop and message dispatch */
#include "iointernal.h"
#include "iomessage.h"

fs_message(pid_t pid, fs_msg *m, unsigned len)
{
	inode_t *root;
	switch (m->type) {
	case _FSYS_OLD_CHDIR:
		reply_errno(pid, ENOSYS);
		break;

	case _FSYS_MKSPECIAL:
		switch (m->mkspec.mode & S_IFMT) {
		case S_IFIFO:
		case S_IFCHR:
			reply_errno(pid, ENOSYS);
			break;
		case S_IFDIR:
			if ((root=getroot(m->mkspec.unit))==0) {
				reply_errno(pid, ENOENT);
			}
			fs_mkdir(pid, m->mkspec.path, root,
			         m->mkspec.mode&S_IPERMS);
			break;
		case S_IFNAM:
		case S_IFBLK:
		case S_IFREG:
		case S_IFLNK:
		case S_IFSOCK:
		default:
			reply_errno(pid, ENOSYS);
		}
		break;

	case _FSYS_REMOVE:
		if ((root=getroot(m->unlink.unit))==0) {
			reply_errno(pid, ENOENT);
		}
		fs_unlink(pid, m->unlink.path, root);
		break;

	case _FSYS_LINK:
	case _FSYS_MOUNT_RAMDISK:
	case _FSYS_UNMOUNT_RAMDISK:
	case _FSYS_BLOCK_READ:
	case _FSYS_BLOCK_WRITE:
	case _FSYS_DISK_GET_ENTRY:
	case _FSYS_SYNC:
	case _FSYS_MOUNT_PART:
	case _FSYS_MOUNT:
	case _FSYS_GET_MOUNT:
	case _FSYS_DISK_SPACE:
	case _FSYS_PIPE:
	case _FSYS_TRUNC:
	case _FSYS_OLD_MOUNT_DRIVER:
	case _FSYS_XSTAT:
	case _FSYS_MOUNT_EXT_PART:
	case _FSYS_UMOUNT:
	case _FSYS_RESERVED:
	case _FSYS_READLINK:
	case _FSYS_MOUNT_DRIVER:
	case _FSYS_FSYNC:
	case _FSYS_INFO:
	case _FSYS_FDINFO:
	case _FSYS_MOUNT_DRIVER32:
	default:
		reply_errno(pid, ENOSYS);
	}
	return 0;
}
