/*- namei() - turn a name into an inode
 *
 * BUGS:
 *   1. "x/../x/y" isn't handled; names must be properly resolved.
 *
 */
#include "iointernal.h"


static char *
skipslash(char *name)
{
	do {
		while (*name == '/')
			name++;
	} while (*name == '.' && name[1] == '/');
	return name;
}

static
component(char **path, char *name, int maxlen)
{
	int      i=0;
	char    *p=*path;
	for (i=0,p=*path; i < maxlen && *p && *p!='/'; *name++=*p++,i++);
	if (i < maxlen) {
		*name='\0';
		*path = skipslash(p);
		return EOK;
	} else {
		return ENAMETOOLONG;
	}
}

int
nameip(proc, path, mode, root, p, comp)
	proc_t   proc;
	char    *path, *comp;
	mode_t   mode;
	inode_t *root;
	inode_t **p;
{
	int             e;
	char           *name;
	uid_t           u;
	gid_t           g;

	ids(proc, &u, &g);
	if (*path == 0) {	/* this is a readdir type request */
		path[0] = '.';
		path[1] = '\0';
	}
	name = skipslash(path);
	*p = root;
	if ((e=component(&name, comp, NAME_MAX)) != EOK) {
		return e;
	}
	while (*name) {
		if ((e=d_lookup(root, comp, p)) != EOK ||
		    (e=chk_perm(u,g,mode,*p))!=EOK) {
			return e;
		}
		root = *p;
		if ((e=component(&name, comp, NAME_MAX)) != EOK) {
			return e;
		}
	}
	return EOK;
}

int
namei(proc_t proc, char *path, mode_t mode, inode_t *root, inode_t **p)
{
	char            comp[NAME_MAX+1]; /*current component*/
	int             e;
	if ((e=nameip(proc, path, FEXEC, root, p, comp)) != EOK) {
		return e;
	}
	if ((e=d_lookup(*p, comp, p)) == EOK) {
		uid_t u;
		gid_t g;
		ids(proc, &u, &g);
		e=chk_perm(u,g,mode, *p);
	}
	return e;
}
