/*- server: main loop and message dispatch */

#include <stdio.h>
#include <i86.h>
#include <sys/fd.h>
#include <sys/sched.h>
#include <sys/osinfo.h>
#include <sys/psinfo.h>
#include <sys/kernel.h>
#include "iointernal.h"
#include "iomessage.h"

time_t __far      *timep;           /* fast grab at time */
struct _fd_ctrl	  *fd_ctrl;

const int PFLAGS = _PPF_SERVER|_PPF_PRIORITY_REC|_PPF_SIGCATCH;


#define MAXP 10
static int   pidtab[MAXP];
static int (*pfunc[MAXP])();
static int   np;

server_addpid(pid_t p, int (*f)())
{
	int       i;
	for (i=0; i < np; i++) {
		if (p == pidtab[i])
			return 0;
	}
	if (np < MAXP) {
		pidtab[np] = p;
		pfunc[np++] = f;
	}
	return 0;
}

server_chkpid(pid_t pid)
{
	int        n;
	for (n=0; n < np; n++) {
		if (pid == pidtab[n]) {
			(pfunc[n])(pid);
			return 1;
		}
	}
	return 0;
}



#define MSG_CLASS(m) ((m)>>8)
void
server()
{
	pid_t          pid;
	message        msg;
	struct _mxfer_entry mx;

	_setmx(&mx, &msg, sizeof msg);
	while (1) {
		if ((pid = Receivemx(0, 1, &mx)) == -1) {
			continue;
		}
		if (server_chkpid(pid)) {
			continue;
		}
		switch (MSG_CLASS(msg.type)) {
		case _SYS_CLASS:
			sys_message(pid, &msg.sys, sizeof msg);
			break;
		case _IO_CLASS:
			io_message(pid, &msg.io, sizeof msg);
			break;
		case _FS_CLASS:
			fs_message(pid, &msg.fs, sizeof msg);
			break;
		case _DEV_CLASS:
			dev_message(pid, &msg.dev, sizeof msg);
			break;
		}
	}
}

server_init()
{
	struct _osinfo osdata;
	if(qnx_osinfo(0, &osdata) == -1) {
		perror("osinfo");
		return -1;
	}
	timep = MK_FP(osdata.timesel, offsetof(struct _timesel, seconds));
	if(qnx_pflags(PFLAGS, PFLAGS, NULL, NULL)) {
		perror("pflags");
		return -1;
	}
	if(!(fd_ctrl = __init_fd(getpid()))) {
		perror("__init_fd");
		return -1;
	}
	qnx_scheduler(0, 0, SCHED_RR, 22, 1); 
	return 0;		
}
